@@ -2,7 +2,7 @@ package JavaScript::V8;
 use strict;
 use warnings;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use JavaScript::V8::Context;
 require XSLoader;
@@ -52,6 +52,27 @@ JavaScript::V8 - Perl interface to the V8 JavaScript engine
       }
   /);
 
+=head1 DIRECTION
+
+Since 2012, the v8 library has changed considerably, both in its build
+system and its API. Additionally, at the time of this writing (Apr 2019),
+the latest version available for Ubuntu is about 3.14, and for Homebrew
+is around 3.28. Meanwhile, the latest "release"(ish) from Google is
+around 7.5.99.
+
+The dramatic API changes mean that backward compatibility with the
+current API will be effectively impossible. Unfortunately, the likelihood
+of security holes in the 7-or-so year old software means it will be
+necessary to keep up with the current version, rather than with the one
+v0.08 of this module supports.
+
+Therefore, the next steps will be to use the excellent L<Alien::Build>
+to make an "alien" module that builds and makes available v8. The current
+L<Alien::V8> is not suitable, since its last release was from 2011.
+
+Contributions of effort will be welcome. Please open an RT, or just C<#v8>
+on C<irc.perl.org> to get involved.
+
 =head1 INSTALLING V8
 
 =head2 Memory notes
